/*
 * Copyright © 2022 DATAMART LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.datamart.kafka.postgres.writer.repository;

import org.springframework.stereotype.Repository;
import ru.datamart.kafka.postgres.writer.model.InsertDataContext;
import ru.datamart.kafka.postgres.writer.model.InsertDataContextId;

import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Supplier;

@Repository
public class InsertDataContextRepository {
    private final Map<InsertDataContextId, InsertDataContext> map = new ConcurrentHashMap<>();

    public InsertDataContext getOrCreate(InsertDataContextId insertDataContextId, Supplier<InsertDataContext> contextSupplier) {
        return map.computeIfAbsent(insertDataContextId, ignored -> contextSupplier.get());
    }

    public Optional<InsertDataContext> get(InsertDataContextId insertDataContextId) {
        return Optional.ofNullable(map.get(insertDataContextId));
    }

    public Optional<InsertDataContext> remove(InsertDataContextId insertDataContextId) {
        return Optional.ofNullable(map.remove(insertDataContextId));
    }
}
